module Towel
  # Helpers to collect the environment-specific details for invocations.
  module Environment
    # Environment variable that, if set, defines the source control management
    # revision that is reported to Towel. See `Invocation.revision` in
    # towel.proto for more details.
    REVISION_ENV = "TOWEL_REVISION".freeze

    # Environment variable that, if set, defines what other environment
    # variables should be redacted.
    REDACT_ENV = "TOWEL_REDACT".freeze

    # Value that is substituted for environment variables that have been
    # redacted.
    REDACTED = "REDACTED".freeze

    # The current platform that Towel is running on.
    def self.platform
      platform = Towel::V1alpha::Platform.new
      uname = Etc.uname
      platform.os = uname[:sysname]
      platform.os_version = uname[:release]
      platform.arch = uname[:machine]
      return platform
    end

    # The hostname of this machine.
    def self.hostname
      Socket.gethostname
    end

    # The OS user that this process is running as.
    def self.user
      Etc.getlogin
    end

    # The current working directory.
    def self.working_dir
      Dir.getwd
    end

    # The arguments used to launch this process.
    def self.argv
      [$0].concat(ARGV)
    end

    # The environment variables set for this process.
    def self.env_vars
      redacted = ENV.fetch(REDACT_ENV, "").split(",").map(&:strip)
      ENV.to_h do |name, value|
        if redacted.include?(name)
          [name, REDACTED]
        else
          [name, value]
        end
      end
    end

    # The version control system (VCS) revision for the code that is running.
    def self.revision
      ENV.fetch(REVISION_ENV, "")
    end
  end
end
